import { CandidateEntity } from '@modules/ad/core/domain/candidate.entity';
import { Completer } from './completer.abstract';
import { MatchQuery } from '../match.query';
import { Step } from '../../../types/step.type';
import { CarpoolPathItem } from '@modules/ad/core/domain/value-objects/carpool-path-item.value-object';
import { RouteResponse } from '../../../ports/georouter-provider.port';

export class RouteCompleter extends Completer {
  protected readonly type: RouteCompleterType;
  constructor(query: MatchQuery, type: RouteCompleterType) {
    super(query);
    this.type = type;
  }

  complete = async (
    candidates: CandidateEntity[],
  ): Promise<CandidateEntity[]> => {
    await Promise.all(
      candidates.map(async (candidate: CandidateEntity) => {
        switch (this.type) {
          case RouteCompleterType.BASIC:
            const basicCandidateRoute = await this._getRoute(candidate, {
              points: true,
              steps: false,
            });
            candidate.setMetrics(
              basicCandidateRoute.distance,
              basicCandidateRoute.duration,
            );
            break;
          case RouteCompleterType.DETAILED:
            const detailedCandidateRoute = await this._getRoute(candidate, {
              points: true,
              steps: true,
            });
            candidate.setSteps(detailedCandidateRoute.steps as Step[]);
            break;
        }
        return candidate;
      }),
    );
    return candidates;
  };

  _getRoute = async (
    candidate: CandidateEntity,
    detailsSettings: { points: boolean; steps: boolean },
  ): Promise<RouteResponse> =>
    this.query.routeProvider.getRoute({
      waypoints: (candidate.getProps().carpoolPath as CarpoolPathItem[]).map(
        (carpoolPathItem: CarpoolPathItem) => carpoolPathItem,
      ),
      detailsSettings: detailsSettings,
    });
}

export enum RouteCompleterType {
  BASIC = 'basic',
  DETAILED = 'detailed',
}
