import { Frequency, Role } from './ad.types';
import { PointProps } from './value-objects/point.value-object';
import { ScheduleItemProps } from './value-objects/schedule-item.value-object';
import { CarpoolPathItemProps } from './value-objects/carpool-path-item.value-object';
import { JourneyProps } from './value-objects/journey.value-object';
import { StepProps } from './value-objects/step.value-object';

// All properties that a Candidate has
export interface CandidateProps {
  role: Role;
  frequency: Frequency;
  driverWaypoints: PointProps[];
  passengerWaypoints: PointProps[];
  driverSchedule: ScheduleItemProps[];
  passengerSchedule: ScheduleItemProps[];
  driverDistance: number;
  driverDuration: number;
  dateInterval: DateInterval;
  carpoolPath?: CarpoolPathItemProps[];
  distance?: number;
  duration?: number;
  steps?: StepProps[];
  journeys?: JourneyProps[];
  spacetimeDetourRatio: SpacetimeDetourRatio;
}

// Properties that are needed for a Candidate creation
export interface CreateCandidateProps {
  id: string;
  role: Role;
  frequency: Frequency;
  driverDistance: number;
  driverDuration: number;
  driverWaypoints: PointProps[];
  passengerWaypoints: PointProps[];
  driverSchedule: ScheduleItemProps[];
  passengerSchedule: ScheduleItemProps[];
  spacetimeDetourRatio: SpacetimeDetourRatio;
  dateInterval: DateInterval;
}

export enum Target {
  START = 'START',
  INTERMEDIATE = 'INTERMEDIATE',
  FINISH = 'FINISH',
  NEUTRAL = 'NEUTRAL',
}

export abstract class Validator {
  abstract validate(): boolean;
}

export type SpacetimeDetourRatio = {
  maxDistanceDetourRatio: number;
  maxDurationDetourRatio: number;
};

export type DateInterval = {
  lowerDate: string;
  higherDate: string;
};
