import {
  registerDecorator,
  ValidationOptions,
  ValidationArguments,
  isISO8601,
} from 'class-validator';

export function IsAfterOrEqual(
  property: string,
  validationOptions?: ValidationOptions,
) {
  return function (object: object, propertyName: string) {
    registerDecorator({
      name: 'isAfterOrEqual',
      target: object.constructor,
      propertyName: propertyName,
      constraints: [property],
      options: validationOptions,
      validator: {
        validate(value: any, args: ValidationArguments) {
          const [relatedPropertyName] = args.constraints;
          const relatedValue = (args.object as any)[relatedPropertyName];
          if (
            !(
              typeof value === 'string' &&
              typeof relatedValue === 'string' &&
              isISO8601(value, {
                strict: true,
                strictSeparator: true,
              }) &&
              isISO8601(relatedValue, {
                strict: true,
                strictSeparator: true,
              })
            )
          )
            return false;
          return new Date(value) >= new Date(relatedValue);
        },
      },
    });
  };
}
